from dataclasses import field
from typing import Any, Dict, List, Optional, Union
from pydantic.dataclasses import dataclass


@dataclass
class GenerateWebRequest:
    """Project data model"""

    prompt: str
    history: Optional[str] = None


@dataclass
class LLMConfig:
    """Data model for LLM Config for Autogen"""
    seed: int = 42
    config_list: List[Dict[str, Any]] = field(
        default_factory=list)  # a list of OpenAI API configurations
    temperature: float = 0
    use_cache: bool = True
    request_timeout: Optional[int] = None


@dataclass
class AgentConfig:
    """Data model for Agent Config for Autogen"""
    name: str
    llm_config: LLMConfig
    human_input_mode: str = "NEVER"
    max_consecutive_auto_reply: int = 10
    is_termination_msg: Union[bool, str] = None
    code_execution_config: Optional[Union[bool, str, Dict[str, Any]]] = None


@dataclass
class AgentFlowSpec:
    """Data model to help flow load agents from config"""
    type: str  # "assistant" | "user_proxy" | "group_chat"
    config: AgentConfig = field(default_factory=AgentConfig)


@dataclass
class FlowConfig:
    """Data model for Flow Config for Autogen"""
    name: str
    driver_agent: AgentConfig
    receiver_agent: Union[AgentConfig, List[AgentConfig]]





from typing import Dict


def parse_token_usage(logged_history: Dict):
    all_usage = []
    all_cost = 0
    all_tokens = 0
    for key in logged_history.keys():
        curr_usage = logged_history[key]["response"]
        all_cost += curr_usage["cost"]
        all_tokens += curr_usage["usage"]["total_tokens"]
        all_usage.append(curr_usage)
    usage = {
        "total_cost": all_cost,
        "total_tokens": all_tokens,
        "usage": all_usage
    }

    return usage



# a manager class that can
# load an autogen flow run an autogen flow and return the response to the client


from typing import Dict
import autogen


class Flow(object):
    def __init__(self) -> None:

        pass

    def run(self, llm_config:):


# a manager class that can
# load an autogen flow run an autogen flow and return the response to the client


from typing import Dict
import autogen
from .utils import parse_token_usage


class Manager(object):
    def __init__(self) -> None:

        pass

    def run_flow(self, prompt: str, flow: str = "default") -> None:
        autogen.ChatCompletion.start_logging(compact=False)
        config_list = autogen.config_list_openai_aoai()

        llm_config = {
            "seed": 42,  # seed for caching and reproducibility
            "config_list": config_list,  # a list of OpenAI API configurations
            "temperature": 0,  # temperature for sampling
            "use_cache": True,  # whether to use cache
        }

        assistant = autogen.AssistantAgent(
            name="assistant",
            max_consecutive_auto_reply=3, llm_config=llm_config,)

        # create a UserProxyAgent instance named "user_proxy"
        user_proxy = autogen.UserProxyAgent(
            name="user_proxy",
            human_input_mode="NEVER",
            llm_config=llm_config,
            max_consecutive_auto_reply=3,
            is_termination_msg=lambda x: x.get("content", "").rstrip().endswith("TERMINATE"),
            code_execution_config={
                "work_dir": "scratch/coding",
                "use_docker": False
            },
        )
        user_proxy.initiate_chat(
            assistant,
            message=prompt,
        )

        messages = user_proxy.chat_messages[assistant]
        logged_history = autogen.ChatCompletion.logged_history
        autogen.ChatCompletion.stop_logging()
        response = {
            "messages": messages[1:],
            "usage": parse_token_usage(logged_history)
        }
        return response



import logging
from typing import Dict
from ..datamodel import GenerateWebRequest
from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles
import os
from fastapi.middleware.cors import CORSMiddleware
import os

from ..manager import Manager
import traceback

logger = logging.getLogger("autogenui")


app = FastAPI()
# allow cross origin requests for testing on localhost: 800 * ports only
app.add_middleware(
    CORSMiddleware,
    allow_origins=["http://localhost:8000", "http://localhost:8001", "http://localhost:3000"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

api = FastAPI(root_path="/api")
app.mount("/api", api)


root_file_path = os.path.dirname(os.path.abspath(__file__))
files_static_root = os.path.join(root_file_path, "files/")
static_folder_root = os.path.join(root_file_path, "ui")


os.makedirs(files_static_root, exist_ok=True)
api.mount("/files", StaticFiles(directory=files_static_root, html=True), name="files")


app.mount("/", StaticFiles(directory=static_folder_root, html=True), name="ui")


manager = Manager()


@api.post("/generate")
async def generate(req: GenerateWebRequest) -> Dict:
    """Generate a response from the autogen flow"""
    prompt = req.prompt
    history = req.history or ""

    prompt = f"{history}\n\n{prompt}"
    print("******history******", history)

    try:
        agent_response = manager.run_flow(prompt=prompt)
        response = {
            "data": agent_response,
            "status": True
        }
    except Exception as e:
        traceback.print_exc()
        response = {
            "data": str(e),
            "status": False
        }

    return response


@api.post("/hello")
async def hello() -> None:
    return "hello world"
